//INTERBANK MARKET MODEL
// Oscar Arce, Galo Nuo, Dominik Thaler, Carlos Thomas
// March 2017

// VARIABLES
var SDF Y Q I C K N W L mu pi pstar Xi1 Xi2 Delta Ra Rk Rlf  Rb Rl  Rdf   Gammab Gammal Phib Phil;
var omegab omegal Bcb Bgcb Bg  D Omega Fomegab Fomegal condexpwb condexpwl condexpwlb Z lambda phi Rstar Rmin Rreal Rg Qg varphi Welfare U Rd eBeta Rib;
varexo errZ errBgcgNP errlam errphi errBgcgRI errN errMP errBeta;

// PARAMETERS
parameters KSS zlb rho omegalSS omegabSS Match lambdaSS lambda2 nu kappa gamma psi  beta delta alpha theta epsilon phiSS xi Rbar pibar chi Bbar mudist sigdist Bcbbar Bgcbbar betab zeta vartheta GammabSS taylor extH elb QEtype nominal multiround lognorm;
alpha   = .33;           % capital share
delta   = .025;          % depreciation
gamma   = 2;             % risk aversion HH
psi     = 1.45;          % labor elasticity
theta   = .7;            % price stickiness
epsilon = 5;             % markup
kappa   = 2.4;           % capital adjustment costs (divided by two)
nu      = 1.5;           % Taylor parameter on inflation
rho     = 0.8;           % taylor rule smoothing
pibar   = 0;             % SS inflation
extH    = 0;             % external habits

beta    = .995;          % discount factor HH
zeta    = .05; //0.05    % Decay rate of bond. lim 1 for 1 period bonds 
Bcbbar  = 0;             % SS MROs (not used in the model)

%GK params
alpha   = .33;           % capital share
delta   = .025;          % depreciation
gamma   = 1;             % risk aversion HH
psi     = 0.276;          % labor elasticity
theta   = .779;            % price stickiness
epsilon = 4.167;             % markup
kappa   = 1.728;           % capital adjustment costs (divided by two)
nu      = 1.5;           % Taylor parameter on inflation
rho     = 0.8;           % taylor rule smoothing
pibar   = 0;             % SS inflation
extH    = 0;             % external habits

phiSS   = 5;             % leverage
xi      = .5;            % bargaining power borrower
chi     = 0.005;         % corridor width
Bbar    = 2.4;           % Debt to SS GDP
Bgcbbar = 0 ;           % SS share of Govt bond holdings by Central Bank (dont put to close to 0 )
vartheta= 0.998;         % probablility of rentering market after failed negotiation

betab      = .975;       % discount factor Banks
lambda2    = .75;        % Matching function parameter2 (only used in Cobb Douglas)
zlb        = 0;          % Lower bound. Options: 2 for ZLB on Rd 1 for ZLB on Rdf, 0 for inactive
elb        = 1+0.0/400;   % Minimum of the relevant interest rate. ZLB->1
nominal    = 1;

multiround = 0;          % 1=Multiround random search. 0=directed search,. Requires Match=0
Match      = 0;          % Matching function type: 0=den haan (lambda need to be big, for lambda>1000 equals Leontief), 1=Leontief, 2=Cobb Douglas
mudist     = 0;          % initialize mean of distribution of iid shock
sigdist    = .01;        % initialize std of distribution of iid shock	
@#define lognorm = 1
lognorm=1;

taylor     = 0;          % Type of Taylor rule. Options: 0 target Rib, 1 target Rdf
QEtype     =0;           % QE type: 0 bond purchases, 1 LTROS
// SOLVING JOINTLY THE STEADY STATE AND REMAINING PARAMETER VALUES
GammabSS   = 1 - 0.0023;         % 0.023 percent ratio of IB claims over CB excess reserves
lambdaSS   = 225; 
if lambdaSS > 1000 % If the parameter is too big the Leontief approximation is better. Set lambda to 1 because tit will still be used for computations and might give infinity
    GammabSS = 1;
    lambdaSS = 1;
    Match=1;
end

% Numerical finding the SS
findSS_ = @(x)findSS4_1( x,beta,mudist,sigdist,epsilon,vartheta,extH, alpha, gamma, phiSS,psi, xi,chi,delta,Bbar ,Bgcbbar, betab,lambda2,Match,Bcbbar,lambdaSS,GammabSS,multiround,lognorm,0);
% Vector x = [omegal omegab K   mudist  sigdist]

lb      = [0.0001 0.0001 0.1 -Inf 0.0001 -Inf 0.0001 0.0001  ]; % lower bounds on x
ub      = [Inf Inf inf inf inf inf inf inf ];       % upper bounds on x
start   = [.99 1 10 1-lognorm 0.02 NaN Bbar phiSS];             % initial values on x
sol     = fmincon(@(x)0,start,[],[],[],[],lb,ub,findSS_,optimset('TolCon',1e-10));
oo_.start=sol(1:3);

% Remaining parameter values
mudist   = sol(4);
sigdist  = sol(5);
Bbar     = sol(7);
phiSS    = sol(8);
KSS      = sol(3);
omegalSS = sol(1);
omegabSS = sol(2);
if  isnan(sol(6))==0
    lambdaSS=sol(6);
end
sol2=findSS4_1( sol,beta,mudist,sigdist,epsilon,vartheta,extH, alpha, gamma, phiSS,psi, xi,chi,delta,Bbar ,Bgcbbar, betab,lambda2,Match,Bcbbar,lambdaSS,GammabSS,multiround,lognorm,1);
Rbar=sol2;

//MAIN MODEL
model;  
    //Households (1-5)
    1   = SDF *Rd /(1+pi(+1))^nominal;
    SDF = eBeta*beta*(C(+1)-extH*C)^(-gamma)/(C-extH*C(-1))^(-gamma);
    W   = L^psi / (C-extH*C(-1))^(-gamma);
    1   = Q* (1-kappa*(I/I(-1)-1)^2 - kappa*2*(I/I(-1)-1)*I/I(-1))+  SDF*Q(+1)*kappa*2*(I(+1)/I-1)*(I(+1)/I)^2; 
    K   =  (1-kappa*(I/I(-1)-1)^2)*I+ (1-delta)*Omega(-1)*K(-1);

    //Firms (6-14)
    Y     = Z/Delta*L^(1-alpha)*(K(-1)*Omega(-1))^alpha;
    1     = theta*(1+pi)^(epsilon-1) + (1-theta)*pstar^(1-epsilon);
    pstar = Xi1/Xi2;
    Xi1   = epsilon/(epsilon-1)*mu*Y   + theta*SDF*(1+pi(+1))^epsilon    *Xi1(+1);
    Xi2   = Y                          + theta*SDF*(1+pi(+1))^(epsilon-1)*Xi2(+1);
    Delta = (1-theta)*pstar^(-epsilon) + theta     *(1+pi)^epsilon       *Delta(-1);
    Ra    = (Rk+(1-delta)*Q)/Q(-1);
    Rk    = alpha*mu*Z*(((1-alpha)*mu*Z)/W)^((1-alpha)/alpha);
    L     = (((1-alpha)*Z*mu)/W)^(1/alpha)*Omega(-1)*K(-1);

    //Banks (15-20) (note: for unknown reasons dynare does not accept replacing the LHS with Rg(+1) in the LHS=Rl equation 
    K*Q    =  N*phi*(1- Fomegab)+(Fomegab-Fomegal)*(N+D);
    N      =  (errN==0)*betab*( Ra*Q(-1)*Omega(-1)*K(-1) - Rb(-1)/(1+pi)*(1-Fomegab(-1))*((phi(-1)-1)*N(-1)-D(-1)) + Rl(-1)/(1+pi)*(Fomegal(-1)*(N(-1)+D(-1))-Bg(-1))+Rg/(1+pi)*Bg(-1)-Rd(-1)/(1+pi)^nominal*D(-1))
             +(errN>0)*errN ;
    omegab =  Rb/(Ra(+1)*(1+pi(+1)));
    omegal =  Rl/(Ra(+1)*(1+pi(+1)));
    Rl     =  (zeta+(1-zeta)*Qg(+1))/Qg;
    Rd/(1+pi(+1))^nominal     =  Rb/(1+pi(+1))  *(1-Fomegab)+Rl/(1+pi(+1)) *Fomegal+(Fomegab-Fomegal)*Ra(+1)*condexpwlb;

    //Interbank market (21-27)
    Phib   =  (N*(phi-1)-D)*(1-Fomegab)-Bcb;
    Phil   =  (N+D)*Fomegal-Bg;
    Gammab =  (Match==1)*min(Phil/Phib,1)    
             +(Match==0)*Phil/((Phib*2/(Phil+Phib))^lambda+ (Phil*2/(Phil+Phib))^lambda)^(1/lambda)*(2/(Phil+Phib))     
             +(Match==2)*min(min(1,Phil/Phib),lambda2*(Phil/Phib)^lambda);
    Gammal =   min((Match==1)+(lambda>1000),1)*Phib/Phil           +      ((Match==0)*(lambda<=1000)*(lambda>1))*Phib/((Phib*2/(Phil+Phib))^lambda+ (Phil*2/(Phil+Phib))^lambda)^(1/lambda)*(2/(Phil+Phib))     +    (Match==2)*min(min(1,Phib/Phil),lambda2*(Phib/Phil)^(1-lambda));
    Rb     =  Gammab*varphi*Rdf+(1-varphi*Gammab)*Rlf;
    Rl     =  (1-varphi)*Gammal*Rlf+(1-(1-varphi)*Gammal)*Rdf;
    varphi =  (multiround==1)*(xi*(1-vartheta*Gammal))/(1-xi*vartheta*Gammal-(1-xi)*vartheta*Gammab)
             +(multiround==0)*(1/((Phib/Phil)^lambda+1));

    //Central bank (28-34)
    Rstar                        =  (taylor==1) * (rho*Rstar(-1)+(1-rho)*(steady_state(Rdf)+nu*(pi-pibar)))
                                   +(taylor==0) * (rho*(Rstar(-1)+(1-varphi(-1))*chi)+(1-rho)*((1-steady_state(varphi))*steady_state(Rlf) +steady_state(varphi)*steady_state(Rdf)+nu*(pi-pibar))-(1-varphi)*chi );
    Rmin                         =  (errMP<3 )*(zlb==2) * (elb - chi*(Gammab*varphi - 1)*(Fomegab - 1) - Ra(+1)*condexpwlb*(Fomegab - Fomegal)*(pi(+1) + 1) + Fomegal*Gammal*chi*(varphi - 1))/(Fomegal - Fomegab + 1)
                                   +(errMP<3 )*(zlb==1) * elb
                                   +(errMP==3)*(zlb==2) * (-(-1 +(1-varphi*Gammab*Rlf*(1-Fomegab))+Rl*Fomegal+(Fomegab-Fomegal)*Ra(+1)*(1+pi(+1))*condexpwlb)/ (Gammab*varphi*(1-Fomegab)))
                                   +(errMP==3)*(zlb==1) * 1;
    Rdf                          =  (errMP==0) * max(Rstar,Rmin) //(max(Rstar,Rmin) +log(1+exp((min(Rstar,Rmin) -max(Rstar,Rmin))))/100000) 
                                   +(errMP==1) * Rmin  
                                   +(errMP==2) * Rstar 
                                   +(errMP==3) * Rmin
                                   +(errMP<2-(errMP==1))*errMP; 



  //  [mcp = 'Rdf > 1.0011']
  //  Rdf=Rstar;
    Rdf                          = Rlf-chi;
    Bgcb + Bcb + Phib*(1-Gammab) = Phil*(1-Gammal);
    Bgcb                         = Bgcb(-1)*(1-zeta)+zeta*steady_state(Y)*Bbar*Bgcbbar+steady_state(Y)*Bbar*errBgcgNP*(QEtype==0)+(Bgcb(-1)-steady_state(Bgcb))*zeta*errBgcgRI*(QEtype==0); % QE  original!
    //Bgcb                         = (Bgcb(-1)*(1-zeta)+zeta*steady_state(Y)*Bbar*Bgcbbar+steady_state(Y)*Bbar*errBgcgNP*(QEtype==0)+(Bgcb(-1)-steady_state(Bgcb))*zeta*errBgcgRI*(QEtype==0))*(Rstar>Rmin)+(Rstar<=Rmin)*0.2*steady_state(Bgcb); 
    //0=(Bgcb-steady_state(Bgcb))*(Rstar==Rdf)+(Bgcb-0.2*steady_state(Bgcb))*(Rdf==Rmin);
    //0=(Bgcb-steady_state(Bgcb))*(Rstar>=Rdf)+(Rstar-Rmin)*(Rdf==Rmin);
    //Bgcb                       = Y*Bbar*Bgcbbar+(steady_state(Y)*Bbar*errBgcgNP+(Bgcb(-1)-steady_state(Bgcb))*zeta*errBgcgRI)*(QEtype==0); % QE 
    Bcb                          = Bcb(-1)*(1-zeta)+zeta*steady_state(Y)*Bbar*Bcbbar+steady_state(Y)*Bbar*errBgcgNP*(QEtype==1)+(Bcb(-1)-steady_state(Bcb))*zeta*errBgcgRI*(QEtype==1);
  
   // Government (35-36)
    Bbar*steady_state(Y)         = Bgcb+Bg; 
    Rg                           = (zeta+(1-zeta)*Qg)/(Qg(-1));

    //Auxiliaries for banks
@#if lognorm
    condexpwb  =exp(mudist+sigdist^2/2)*normcdf((-log(omegab)+mudist+sigdist^2)/sigdist)/(.5-.5*erf((log(omegab)-mudist)/(sqrt(2)*sigdist)));
    condexpwl  =exp(mudist+sigdist^2/2)*normcdf((-log(omegal)+mudist+sigdist^2)/sigdist)/(.5-.5*erf((log(omegal)-mudist)/(sqrt(2)*sigdist)));
    condexpwlb =((condexpwl*(1-Fomegal)-condexpwb*(1-Fomegab)))/((Fomegab-Fomegal)+0*(abs(omegab-omegal)<1e-7));
    Fomegab  = normcdf((log(omegab)-mudist)/sigdist);
    Fomegal  = normcdf((log(omegal)-mudist)/sigdist);
@#else
    condexpwb  = (omegab+(mudist+sigdist))/2;
    condexpwl  = (omegal+(mudist+sigdist))/2;
    condexpwlb = (omegal+omegab)/2;
    Fomegab    = (omegab-(mudist-sigdist))/(2*sigdist);
    Fomegal    = (omegal-(mudist-sigdist))/(2*sigdist);
@#endif

    //Aggregate constraints
    Omega = (phi*(1-Fomegab)*condexpwb +(Fomegab-Fomegal)*condexpwlb*(1+D/N))/(phi*(1-Fomegab)+(Fomegab-Fomegal)*(1+D/N));
    //Omega = steady_state(Omega);
    Y     = C+I;

    //Shocks
    Z      = exp(errZ);
    lambda = lambdaSS*exp(errlam);
    phi    = phiSS*exp(errphi); 
    log(eBeta)=0.9*log(eBeta(-1))+(errBeta);

    // Extra definitions
    Rreal     = Rd/(1+pi(+1))^nominal;
    U         = (gamma>1)*(C-extH*C(-1))^(1-gamma)/(1-gamma+(gamma==1))+(gamma==1)*log(C-extH*C(-1))-L^(1+psi)/(1+psi);
    Welfare   = U+beta*Welfare(+1);
    Rib       = varphi*Rdf+(1-varphi)*Rlf;

end;

// STEADY STATE
steady_state_model ;
    omegal=SS4_1( beta,mudist,sigdist,epsilon,vartheta,extH, alpha, gamma, phiSS,psi, xi,chi,delta,Bbar ,Bgcbbar, betab,lambda2,Match,Bcbbar,lambdaSS,GammabSS,multiround,lognorm,1);
    omegab=SS4_1( beta,mudist,sigdist,epsilon,vartheta,extH, alpha, gamma, phiSS,psi, xi,chi,delta,Bbar ,Bgcbbar, betab,lambda2,Match,Bcbbar,lambdaSS,GammabSS,multiround,lognorm,2);
    K     =SS4_1( beta,mudist,sigdist,epsilon,vartheta,extH, alpha, gamma, phiSS,psi, xi,chi,delta,Bbar ,Bgcbbar, betab,lambda2,Match,Bcbbar,lambdaSS,GammabSS,multiround,lognorm,3);
    Rdf   =SS4_1( beta,mudist,sigdist,epsilon,vartheta,extH, alpha, gamma, phiSS,psi, xi,chi,delta,Bbar ,Bgcbbar, betab,lambda2,Match,Bcbbar,lambdaSS,GammabSS,multiround,lognorm,4); 
    % notice: findSS gives us Rdf. Rbar is defined as steadystate(Rdf) idependently of the taylor rule.
    phi    = phiSS;

@#if lognorm
    condexpwb = exp(mudist+sigdist^2/2)*normcdf((-log(omegab)+mudist+sigdist^2)/sigdist)/(.5-.5*erf((log(omegab)-mudist)/(sqrt(2)*sigdist)));
    condexpwl = exp(mudist+sigdist^2/2)*normcdf((-log(omegal)+mudist+sigdist^2)/sigdist)/(.5-.5*erf((log(omegal)-mudist)/(sqrt(2)*sigdist)));
    condexpwlb = (condexpwl*(1-normcdf((log(omegal)-mudist)/sigdist))-condexpwb*(1-normcdf((log(omegab)-mudist)/sigdist)))/((1-normcdf((log(omegal)-mudist)/sigdist)-(1-normcdf((log(omegab)-mudist)/sigdist)))+0*(abs(omegab-omegal)<1e-7));
    Fomegab    = normcdf((log(omegab)-mudist)/sigdist);
    Fomegal    = normcdf((log(omegal)-mudist)/sigdist);
@#else
    condexpwb  = (omegab+(mudist+sigdist))/2;
    condexpwl  = (omegal+(mudist+sigdist))/2;
    condexpwlb = (omegal+omegab)/2;
    Fomegab    = (omegab-(mudist-sigdist))/(2*sigdist);
    Fomegal    = (omegal-(mudist-sigdist))/(2*sigdist);
@#endif


    
    Omega=1;
    Rd=1/beta;
    I=K*(1-(1-delta)*Omega);
    Ra= Rd /(omegab*(1-Fomegab)+omegal*Fomegal+(Fomegab-Fomegal)*condexpwlb);
    Rb= omegab*Ra;
    Rl= omegal*Ra ;
    Rk=Ra-(1-delta);
    mu=(epsilon-1)/epsilon;
    W=(1-alpha)*mu*(Rk/(alpha*mu))^(alpha/(alpha-1));
    L=(((1-alpha)*mu/W)^(1/alpha))*Omega*K;
    Y=(Omega*K)^alpha*L^(1-alpha);
    Bgcb= Bgcbbar*Y*Bbar;
    Bg  = Bbar*Y-Bgcb;
    D=(K + (Fomegal + Fomegab *(-1 + phiSS) - phiSS)*(Bg + K + Bbar *Bgcbbar *Y))/((-1 + Fomegab)* phiSS);
    N=(K-D*(Fomegab-Fomegal))/(phiSS*(1- Fomegab)+(Fomegab-Fomegal)) ; 
    C=Y-I;
    Phib=(1-Fomegab)*(N*(phiSS-1)-D);
    Phil=Bgcbbar*Bbar*Y+Phib;
    m=(Match==1)*min(Phil,Phib)+(Match==0)*Phil*Phib/((Phil^lambdaSS+Phib^lambdaSS)^(1/lambdaSS))+(Match==2)*min(min(Phib,Phil),lambda2*Phil^lambdaSS*Phib^(1-lambdaSS));
    Gammab=m/Phib;
    Gammal=m/Phil;
    varphi=(multiround==1)*(xi*(1-vartheta*Gammal))/(1-xi*vartheta*Gammal-(1-xi)*vartheta*Gammab)+(multiround==0)*(1/((Phib/Phil)^lambdaSS+1));
    Bcb    = Bcbbar;
    Z      = 1;
    Q      = 1;
    Delta  = 1;
    pstar  = 1;
    pi     = 0;
    SDF    = beta*C^(-gamma)/C^(-gamma);
    Rlf    = Rdf+chi;           
    Xi1    = epsilon/(epsilon-1)*mu*Y/(1-theta*SDF);
    Xi2    = Y/(1-theta*SDF);
    lambda = lambdaSS;
    Rstar  = Rdf;             
    Rreal  = Rd/(1+pi);
    Rg     = Rl;
    Qg     = zeta/(Rg-(1-zeta));
    U      = (gamma>1)*(C-extH*C)^(1-gamma)/(1-gamma+(gamma==1))+(gamma==1)*log(C-extH*C)-L^(1+psi)/(1+psi);
    Welfare= U/(1-beta);
    Rmin   = (zlb==2)*((elb - chi*(Gammab*varphi - 1)*(Fomegab - 1) - Ra*condexpwlb*(Fomegab - Fomegal)*(pi + 1) + Fomegal*Gammal*chi*(varphi - 1))/(Fomegal - Fomegab + 1))+(zlb==1)*elb;
    eBeta=1;
    Rib       = varphi*Rdf+(1-varphi)*Rlf;

end;

model_diagnostics;

steady;
check;

// SHOCKS
shocks;
    var errZ;    % TFP
    periods 1; 
    values  0;
    var errBgcgNP; % QE
    periods 1:10; 
    values 0 ;
    var errBgcgRI; % QE
    periods 1; 
    values 0 ;
    var errlam;  % Matching efficiency
    periods 0:12 ; 
    values 0 ;
    var errphi;  % Deleveraging
    periods 1;//1:12 13 14 15 16; 
    values 0;//-.33 -.27 -.20 -.13 -.06  ;//-0.28 -.15 0;
    var errMP;   % Controls whether the ZLB is binding
    periods 1:4 5:20 ; 
    values  0 0;//-0.005 0;
    var errN;    % recapitalize banks
    periods 0; 
    values 0;// 8.270757781004315;
    var errBeta;    % impacience
    periods 1; 
    values 0.00; //0.00864; // 0.00363
   end;

// SOLUTION OF THE MODEL
options_.debug=1;

